@@ -74,7 +74,6 @@ static GObjectClass *parent_class;
 static void mime_edit_dialog_init        (MimeEditDialog *mime_edit_dialog,
 					  MimeEditDialogClass *class);
 static void mime_edit_dialog_class_init  (MimeEditDialogClass *class);
-static void mime_edit_dialog_base_init   (MimeEditDialogClass *class);
 
 static void mime_edit_dialog_set_prop    (GObject        *object, 
 					  guint           prop_id,
@@ -106,6 +105,7 @@ static void response_cb                  (MimeEditDialog *dialog,
 
 static void update_sensitivity           (MimeEditDialog *dialog);
 
+
 GType
 mime_edit_dialog_get_type (void)
 {
@@ -114,7 +114,7 @@ mime_edit_dialog_get_type (void)
 	if (!mime_edit_dialog_type) {
 		GTypeInfo mime_edit_dialog_info = {
 			sizeof (MimeEditDialogClass),
-			(GBaseInitFunc) mime_edit_dialog_base_init,
+			(GBaseInitFunc) NULL,
 			NULL, /* GBaseFinalizeFunc */
 			(GClassInitFunc) mime_edit_dialog_class_init,
 			NULL, /* GClassFinalizeFunc */
@@ -134,6 +134,31 @@ mime_edit_dialog_get_type (void)
 	return mime_edit_dialog_type;
 }
 
+/**
+ * mime_edit_editable_enters: Make the "activate" signal of an editable click
+ * the default dialog button.
+ * @dialog: dialog to affect.
+ * @editable: Editable to affect.
+ *
+ * This is a literal copy of gnome_dialog_editable_enters, but not restricted
+ * to GnomeDialogs.
+ *
+ * Normally if there's an editable widget (such as #GtkEntry) in your
+ * dialog, pressing Enter will activate the editable rather than the
+ * default dialog button. However, in most cases, the user expects to
+ * type something in and then press enter to close the dialog. This
+ * function enables that behavior.
+ *
+ **/
+static void
+mime_edit_editable_enters (MimeEditDialog *dialog, GtkEditable *editable)
+{
+	g_signal_connect_swapped (G_OBJECT (editable),
+		"activate",
+		G_CALLBACK (gtk_window_activate_default),
+		GTK_WINDOW (dialog->p->dialog_win));
+}
+
 static void
 mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 {
@@ -165,9 +190,11 @@ mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 
 	dialog->p->dialog_win = gtk_dialog_new_with_buttons
 		(_("Edit file type"), NULL, -1,
-		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog->p->dialog_win),
+		 GTK_RESPONSE_OK);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog->p->dialog_win)->vbox), WID ("edit_widget"), TRUE, TRUE, 0);
 
@@ -179,11 +206,10 @@ mime_edit_dialog_init (MimeEditDialog *dialog, MimeEditDialogClass *class)
 				  (GCallback) use_category_defaults_toggled_cb, dialog);
 
 	g_signal_connect_swapped (G_OBJECT (dialog->p->dialog_win), "response", (GCallback) response_cb, dialog);
-}
 
-static void
-mime_edit_dialog_base_init (MimeEditDialogClass *class) 
-{
+	mime_edit_editable_enters (dialog, GTK_EDITABLE (WID ("description_entry")));
+	mime_edit_editable_enters (dialog, GTK_EDITABLE (WID ("mime_type_entry")));
+	mime_edit_editable_enters (dialog, GTK_EDITABLE (WID ("category_entry")));
 }
 
 static void
@@ -262,8 +288,11 @@ mime_edit_dialog_set_prop (GObject *object, guint prop_id, const GValue *value,
 		mime_edit_dialog->p->is_add = g_value_get_boolean (value);
 
 		if (mime_edit_dialog->p->is_add) {
-			mime_edit_dialog->p->info = mime_type_info_new (NULL, NULL);
+			mime_edit_dialog->p->info = mime_type_info_new (NULL,
+				mime_edit_dialog->p->model);
 			setup_add_dialog (mime_edit_dialog);
+			gtk_window_set_title (GTK_WINDOW (mime_edit_dialog->p->dialog_win),
+				(_("Add file type")));
 			gtk_widget_show_all (mime_edit_dialog->p->dialog_win);
 		}
 
@@ -348,12 +377,19 @@ mime_edit_dialog_new (GtkTreeModel *model, MimeTypeInfo *info)
 }
 
 GObject *
-mime_add_dialog_new (GtkTreeModel *model) 
+mime_add_dialog_new (GtkTreeModel *model, GtkWindow *parent) 
 {
-	return g_object_new (mime_edit_dialog_get_type (),
-			     "model", model,
-			     "is-add", TRUE,
-			     NULL);
+	GObject *obj = g_object_new (mime_edit_dialog_get_type (),
+		"model", model,	/* must be before is-add */
+		NULL);
+	g_object_set (obj,
+		      "is-add", TRUE,
+		      NULL);
+	if (parent != NULL)
+		gtk_window_set_transient_for (
+			GTK_WINDOW (MIME_EDIT_DIALOG (obj)->p->dialog_win),
+			parent);
+	return obj;
 }
 
 static void
@@ -386,9 +422,6 @@ fill_dialog (MimeEditDialog *dialog)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("use_category_defaults_toggle")), dialog->p->use_cat_dfl);
 	update_sensitivity (dialog);
 
-	if (dialog->p->info->mime_type != NULL && *dialog->p->info->mime_type != '\0')
-		gtk_widget_set_sensitive (WID ("mime_type_entry"), FALSE);
-
 	gnome_icon_entry_set_filename (GNOME_ICON_ENTRY (WID ("icon_entry")), mime_type_info_get_icon_path (dialog->p->info));
 
 	populate_component_list (dialog);
@@ -669,7 +702,7 @@ store_data (MimeEditDialog *dialog)
 	}
 	g_free (tmp);
 
-	model_entry_append_to_dirty_list (MODEL_ENTRY (dialog->p->info));
+	model_entry_save (MODEL_ENTRY (dialog->p->info));
 
 	if (!cat_changed) {
 		mime_types_model_construct_iter (MIME_TYPES_MODEL (dialog->p->model),
@@ -684,31 +717,30 @@ static gboolean
 validate_data (MimeEditDialog *dialog) 
 {
 	const gchar *tmp;
-	GtkWidget *err_dialog = NULL;
+	const gchar *mesg = NULL;
 
 	tmp = gtk_entry_get_text (GTK_ENTRY (WID ("mime_type_entry")));
 
 	if (tmp != NULL && *tmp != '\0') {
 		if (strchr (tmp, ' ') || !strchr (tmp, '/')) {
-			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->p->dialog_win),
-							     0, GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_OK,
-							     _("Invalid MIME type. Please enter a valid MIME type, or "
-							       "leave the field blank to have one generated for you."));
-		}
-		else if (dialog->p->is_add && (gnome_vfs_mime_type_is_known (tmp) || get_mime_type_info (tmp) != NULL)) {
-			err_dialog = gtk_message_dialog_new (GTK_WINDOW (dialog->p->dialog_win),
-							     0, GTK_MESSAGE_ERROR,
-							     GTK_BUTTONS_OK,
-							     _("There already exists a MIME type of that name."));
+			mesg = _("Invalid MIME type. Please enter a valid MIME type, or "
+				 "leave the field blank to have one generated for you.");
+		} else if (dialog->p->is_add && (gnome_vfs_mime_type_is_known (tmp) ||
+						 get_mime_type_info (tmp) != NULL)) {
+			mesg = _("There already exists a MIME type of that name.");
 		}
 	}
 
-	if (err_dialog != NULL) {
-			gtk_window_set_modal (GTK_WINDOW (err_dialog), TRUE);
+	if (mesg != NULL) {
+		GtkWidget *err_dialog = gtk_message_dialog_new (
+			GTK_WINDOW (dialog->p->dialog_win),
+			GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR,
+			GTK_BUTTONS_OK,
+			mesg);
 		gtk_dialog_run (GTK_DIALOG (err_dialog));
-			return FALSE;
-		}
+		gtk_object_destroy (GTK_OBJECT (err_dialog));
+		return FALSE;
+	}
 	return TRUE;
 }
 
@@ -771,8 +803,8 @@ choose_cat_cb (MimeEditDialog *dialog)
 
 	dialog_win = gtk_dialog_new_with_buttons
 		(_("Choose a file category"), NULL, -1,
-		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+		 GTK_STOCK_OK,     GTK_RESPONSE_OK,
 		 NULL);
 
 	gtk_widget_set_size_request (dialog_win, 300, 300);
