@@ -34,7 +34,6 @@
 #include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <gconf/gconf-client.h>
-#include <ctype.h>
 
 #include "libuuid/uuid.h"
 
@@ -201,7 +200,7 @@ mime_type_info_using_custom_app (const MimeTypeInfo *info)
 	gchar *tmp;
 	gboolean ret;
 
-	if (info->default_action->name == NULL)
+	if (!info->default_action || !info->default_action->name)
 		return TRUE;
 
 	tmp = g_strdup_printf ("Custom %s", info->mime_type);
@@ -230,7 +229,7 @@ mime_type_info_set_category_name (const MimeTypeInfo *info, const gchar *categor
 void
 mime_type_info_set_file_extensions (MimeTypeInfo *info, GList *list)
 {
-	/* FIXME: Free the old list */
+	gnome_vfs_mime_extensions_list_free (info->file_extensions);
 	info->file_extensions = list;
 }
 
@@ -241,10 +240,13 @@ mime_type_info_save (const MimeTypeInfo *info)
 	uuid_t  app_uuid;
 	gchar   app_uuid_str[100];
 
+	gnome_vfs_mime_freeze ();
 	gnome_vfs_mime_set_description (info->mime_type, info->description);
 	gnome_vfs_mime_set_icon (info->mime_type, info->icon_name);
 
-	if (info->default_action != NULL && info->default_action->command != NULL && *info->default_action->command != '\0') {
+	if (info->default_action != NULL &&
+	    info->default_action->command != NULL &&
+	    info->default_action->command [0] != '\0') {
 		tmp = g_strdup_printf ("Custom %s", info->mime_type);
 
 		if (info->default_action->name == NULL)
@@ -255,11 +257,7 @@ mime_type_info_save (const MimeTypeInfo *info)
 			uuid_unparse (app_uuid, app_uuid_str);
 
 			info->default_action->id = g_strdup (app_uuid_str);
-
-			gnome_vfs_application_registry_save_mime_application (info->default_action);
-			gnome_vfs_application_registry_sync ();
-		}
-		else if (!strcmp (tmp, info->default_action->name)) {
+		} else if (!strcmp (tmp, info->default_action->name)) {
 			gnome_vfs_application_registry_set_value (info->default_action->id, "command",
 								  info->default_action->command);
 			gnome_vfs_application_registry_set_bool_value (info->default_action->id, "requires_terminal",
@@ -267,6 +265,8 @@ mime_type_info_save (const MimeTypeInfo *info)
 		}
 
 		gnome_vfs_mime_set_default_application (info->mime_type, info->default_action->id);
+
+		gnome_vfs_application_registry_save_mime_application (info->default_action);
 	} else {
 		gnome_vfs_mime_set_default_application (info->mime_type, NULL);
 	}
@@ -285,6 +285,9 @@ mime_type_info_save (const MimeTypeInfo *info)
 	g_free (tmp);
 
 	gnome_vfs_mime_set_value (info->mime_type, "use_category_default", info->use_category ? "yes" : "no");
+
+	gnome_vfs_application_registry_sync ();
+	gnome_vfs_mime_thaw ();
 }
 
 void
@@ -333,7 +336,8 @@ get_gconf_base_name (MimeCategoryInfo *category)
 	tmp1 = mime_category_info_get_full_name (category);
 
 	for (tmp = tmp1; *tmp != '\0'; tmp++)
-		if (isspace (*tmp) || *tmp == '(' || *tmp == ')') *tmp = '-';
+		if (g_ascii_isspace (*tmp) || *tmp == '(' || *tmp == ')')
+			*tmp = '-';
 
 	tmp = g_strconcat ("/desktop/gnome/file-types-categories/", tmp1, NULL);
 
@@ -356,7 +360,11 @@ mime_category_info_load_all (MimeCategoryInfo *category)
 
 		if (appid != NULL && *appid != '\0')
 			category->default_action = gnome_vfs_application_registry_get_mime_application (appid);
-		else
+
+		/* This must be non NULL, so be extra careful incase gnome-vfs
+		 * spits back a NULL
+		 */
+		if (category->default_action == NULL)
 			category->default_action = g_new0 (GnomeVFSMimeApplication, 1);
 	}
 
@@ -381,7 +389,9 @@ mime_category_info_using_custom_app (const MimeCategoryInfo *category)
 	gchar *tmp;
 	gboolean ret;
 
-	if (category->default_action->name == NULL)
+	if (category == NULL ||
+	    category->default_action == NULL ||
+	    category->default_action->name == NULL)
 		return TRUE;
 
 	tmp = g_strdup_printf ("Custom %s", category->name);
@@ -470,41 +480,6 @@ mime_category_info_save (MimeCategoryInfo *category)
 		set_subcategory_ids (MODEL_ENTRY (category), category, category->default_action->id);
 }
 
-static void
-update_subcategories (ModelEntry *entry, MimeCategoryInfo *category) 
-{
-	ModelEntry *tmp;
-
-	switch (entry->type) {
-	case MODEL_ENTRY_MIME_TYPE:
-		if (MIME_TYPE_INFO (entry)->use_category) {
-			gnome_vfs_mime_application_free (MIME_TYPE_INFO (entry)->default_action);
-
-			if (category->default_action == NULL)
-				MIME_TYPE_INFO (entry)->default_action = NULL;
-			else
-				MIME_TYPE_INFO (entry)->default_action = gnome_vfs_mime_application_copy (category->default_action);
-		}
-
-		break;
-
-	case MODEL_ENTRY_CATEGORY:
-		if (entry == MODEL_ENTRY (category) || MIME_CATEGORY_INFO (entry)->use_parent_category)
-			for (tmp = entry->first_child; tmp != NULL; tmp = tmp->next)
-				update_subcategories (tmp, category);
-		break;
-
-	default:
-		break;
-	}
-}
-
-void
-mime_category_info_update (MimeCategoryInfo *info) 
-{
-	update_subcategories (MODEL_ENTRY (info), info);
-}
-
 static GList *
 find_possible_supported_apps (ModelEntry *entry, gboolean top) 
 {
@@ -606,7 +581,7 @@ mime_category_info_get_full_name (MimeCategoryInfo *info)
 {
 	GString *string;
 	ModelEntry *tmp;
-	gchar *ret;
+	gchar *ret, *s;
 
 	string = g_string_new ("");
 
@@ -615,7 +590,10 @@ mime_category_info_get_full_name (MimeCategoryInfo *info)
 		g_string_prepend (string, "/");
 	}
 
-	ret = g_strdup ((*string->str == '\0') ? string->str : string->str + 1);
+	/* work around gcc 2.96 bug */
+	s = (*string->str == '\0') ? string->str : string->str + 1;
+	ret = g_strdup (s);
+		
 	g_string_free (string, TRUE);
 	return ret;
 }
@@ -625,7 +603,7 @@ mime_category_info_get_full_description (MimeCategoryInfo *info)
 {
 	GString *string;
 	ModelEntry *tmp;
-	gchar *ret;
+	gchar *ret, *s;
 
 	string = g_string_new ("");
 
@@ -634,7 +612,11 @@ mime_category_info_get_full_description (MimeCategoryInfo *info)
 		g_string_prepend (string, "/");
 	}
 
-	ret = g_strdup ((*string->str == '\0') ? string->str : string->str + 1);
+
+	/* work around gcc 2.96 bug */
+	s = (*string->str == '\0') ? string->str : string->str + 1;
+	ret = g_strdup (s);
+
 	g_string_free (string, TRUE);
 	return ret;
 }
@@ -725,7 +707,7 @@ get_category_name (const gchar *mime_type)
 	else if (!strncmp (mime_type, "audio/", strlen ("audio/")))
 		return "Audio";
 	else
-		return NULL;
+		return "Misc";
 }
 
 static const gchar *
@@ -744,7 +726,7 @@ get_category_description (const gchar *mime_type)
 	else if (!strncmp (mime_type, "audio/", strlen ("audio/")))
 		return _("Audio");
 	else
-		return NULL;
+		return _("Misc");
 }
 
 static GSList *
@@ -880,6 +862,7 @@ static MimeCategoryInfo *
 get_category (const gchar *category_name, const gchar *category_desc, GtkTreeModel *model)
 {
 	ModelEntry *current, *child;
+	gchar **cf = NULL, **df = NULL;
 	gchar **categories = NULL, **desc_categories = NULL;
 	int i;
 
@@ -887,11 +870,9 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 		return NULL;
 
 	if (category_name != NULL)
-		categories = g_strsplit (category_name, "/", -1);
-
+		categories = cf = g_strsplit (category_name, "/", -1);
 	if (category_desc != NULL)
-		desc_categories = g_strsplit (category_desc, "/", -1);
-
+		desc_categories = df = g_strsplit (category_desc, "/", -1);
 	if (category_name == NULL)
 		categories = desc_categories;
 	else if (category_desc == NULL)
@@ -904,7 +885,7 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 			if (child->type != MODEL_ENTRY_CATEGORY)
 				continue;
 
-			if (!strcmp ((category_name == NULL) ?
+			if (!g_ascii_strcasecmp ((category_name == NULL) ?
 				     MIME_CATEGORY_INFO (child)->description :
 				     MIME_CATEGORY_INFO (child)->name,
 				     categories[i]))
@@ -920,8 +901,8 @@ get_category (const gchar *category_name, const gchar *category_desc, GtkTreeMod
 		current = child;
 	}
 
-	g_strfreev (categories);
-	g_strfreev (desc_categories);
+	g_strfreev (cf);
+	g_strfreev (df);
 
 	return MIME_CATEGORY_INFO (current);
 }
