@@ -31,6 +31,7 @@
 #include <gnome.h>
 #include <gconf/gconf-client.h>
 #include <glade/glade.h>
+#include <gdk/gdkx.h>
 #include <math.h>
 
 #include "capplet-util.h"
@@ -98,6 +99,32 @@ double_click_to_gconf (GConfPropertyEditor *peditor, const GConfValue *value)
 	return new_value;
 }
 
+
+static void
+get_default_mouse_info (int *default_numerator, int *default_denominator, int *default_threshold)
+{
+	int numerator, denominator;
+	int threshold;
+	int tmp_num, tmp_den, tmp_threshold;
+	
+	/* Query X for the default value */
+	XGetPointerControl (GDK_DISPLAY (), &numerator, &denominator,
+			    &threshold);
+	XChangePointerControl (GDK_DISPLAY (), True, True, -1, -1, -1);
+	XGetPointerControl (GDK_DISPLAY (), &tmp_num, &tmp_den, &tmp_threshold);
+	XChangePointerControl (GDK_DISPLAY (), True, True, numerator, denominator, threshold);
+
+	if (default_numerator)
+		*default_numerator = tmp_num;
+
+	if (default_denominator)
+		*default_denominator = tmp_den;
+
+	if (default_threshold)
+		*default_threshold = tmp_threshold;
+
+}
+
 static GConfValue *
 motion_acceleration_from_gconf (GConfPropertyEditor *peditor, 
 				const GConfValue *value)
@@ -106,7 +133,17 @@ motion_acceleration_from_gconf (GConfPropertyEditor *peditor,
 	gfloat motion_acceleration;
 
 	new_value = gconf_value_new (GCONF_VALUE_FLOAT);
-	motion_acceleration = CLAMP (gconf_value_get_float (value), 0.2, 6.0);
+	
+	if (gconf_value_get_float (value) == -1.0) {
+		int numerator, denominator;
+		
+		get_default_mouse_info (&numerator, &denominator, NULL);
+
+		motion_acceleration = CLAMP ((gfloat)(numerator / denominator), 0.2, 6.0);
+	}
+	else {
+		motion_acceleration = CLAMP (gconf_value_get_float (value), 0.2, 6.0);
+	}
 
 	if (motion_acceleration >= 1)
 		gconf_value_set_float (new_value, motion_acceleration + 4);
@@ -141,7 +178,17 @@ threshold_from_gconf (GConfPropertyEditor *peditor,
 	GConfValue *new_value;
 
 	new_value = gconf_value_new (GCONF_VALUE_FLOAT);
-	gconf_value_set_float (new_value, CLAMP (gconf_value_get_int (value), 1, 10));
+
+	if (gconf_value_get_int (value) == -1) {
+		int threshold;
+
+		get_default_mouse_info (NULL, NULL, &threshold);
+		gconf_value_set_float (new_value, CLAMP (threshold, 1, 10));
+	}
+	else {
+		gconf_value_set_float (new_value, CLAMP (gconf_value_get_int (value), 1, 10));
+	}
+
 	return new_value;
 }
 
@@ -152,6 +199,26 @@ threshold_to_gconf (GConfPropertyEditor *peditor,
 	return gconf_value_float_to_int (value);
 }
 
+static GConfValue *
+drag_threshold_from_gconf (GConfPropertyEditor *peditor, 
+			   const GConfValue *value)
+{
+	GConfValue *new_value;
+
+	new_value = gconf_value_new (GCONF_VALUE_FLOAT);
+
+	gconf_value_set_float (new_value, CLAMP (gconf_value_get_int (value), 1, 10));
+
+	return new_value;
+}
+
+static GConfValue *
+drag_threshold_to_gconf (GConfPropertyEditor *peditor, 
+		    const GConfValue *value)
+{
+	return gconf_value_float_to_int (value);
+}
+
 /* Retrieve legacy settings */
 
 static void
@@ -164,7 +231,7 @@ get_legacy_settings (void)
 struct test_data_t 
 {
 	gint *timeout_id;
-	GtkWidget *darea;
+	GtkWidget *image;
 };
 
 /* Timeout for the double click test */
@@ -173,7 +240,7 @@ static gboolean
 test_maybe_timeout (struct test_data_t *data)
 {
 	double_click_state = DOUBLE_CLICK_TEST_OFF;
-	gtk_widget_queue_draw (data->darea);
+	gtk_image_set_from_pixbuf (GTK_IMAGE (data->image), double_click_off_pixbuf);
 
 	*data->timeout_id = 0;
 
@@ -182,22 +249,25 @@ test_maybe_timeout (struct test_data_t *data)
 
 /* Callback issued when the user clicks the double click testing area. */
 
-static gint
-drawing_area_button_press_event (GtkWidget      *widget,
-				 GdkEventButton *event,
-				 GConfChangeSet *changeset)
+static gboolean
+event_box_button_press_event (GtkWidget   *widget,
+			      GdkEventButton *event,
+			      GConfChangeSet *changeset)
 {
 	gint                       double_click_time;
 	GConfValue                *value;
-
 	static struct test_data_t  data;
 	static guint               test_on_timeout_id     = 0;
 	static guint               test_maybe_timeout_id  = 0;
 	static guint32             double_click_timestamp = 0;
-
+	GtkWidget                 *image;
+	GdkPixbuf                 *pixbuf = NULL;
+	
 	if (event->type != GDK_BUTTON_PRESS)
 		return FALSE;
 
+	image = g_object_get_data (G_OBJECT (widget), "image");
+	
 	if (!(changeset && gconf_change_set_check_value (changeset, DOUBLE_CLICK_KEY, &value))) 
 		double_click_time = gconf_client_get_int (gconf_client_get_default (), DOUBLE_CLICK_KEY, NULL);
 	else
@@ -211,14 +281,14 @@ drawing_area_button_press_event (GtkWidget      *widget,
 	switch (double_click_state) {
 	case DOUBLE_CLICK_TEST_OFF:
 		double_click_state = DOUBLE_CLICK_TEST_MAYBE;
-		data.darea = widget;
+		data.image = image;
 		data.timeout_id = &test_maybe_timeout_id;
 		test_maybe_timeout_id = gtk_timeout_add (double_click_time, (GtkFunction) test_maybe_timeout, &data);
 		break;
 	case DOUBLE_CLICK_TEST_MAYBE:
 		if (event->time - double_click_timestamp < double_click_time) {
 			double_click_state = DOUBLE_CLICK_TEST_ON;
-			data.darea = widget;
+			data.image = image;
 			data.timeout_id = &test_on_timeout_id;
 			test_on_timeout_id = gtk_timeout_add (2500, (GtkFunction) test_maybe_timeout, &data);
 		}
@@ -229,31 +299,6 @@ drawing_area_button_press_event (GtkWidget      *widget,
 	}
 
 	double_click_timestamp = event->time;
-	gtk_widget_queue_draw (widget);
-
-	return TRUE;
-}
-
-/* Callback to display the appropriate image on the double click testing area. */
-
-static gint
-drawing_area_expose_event (GtkWidget      *widget,
-			   GdkEventExpose *event,
-			   GConfChangeSet *changeset)
-{
-	static gboolean  first_time = TRUE;
-	GdkPixbuf       *pixbuf = NULL;
-
-	if (first_time) {
-		gdk_window_set_events (widget->window, gdk_window_get_events (widget->window) | GDK_BUTTON_PRESS_MASK);
-		g_signal_connect (G_OBJECT (widget), "button_press_event", (GCallback) drawing_area_button_press_event, changeset);
-		first_time = FALSE;
-	}
-
-	gdk_draw_rectangle (widget->window, widget->style->white_gc,
-			    TRUE, 0, 0,
-			    widget->allocation.width,
-			    widget->allocation.height);
 
 	switch (double_click_state) {
 	case DOUBLE_CLICK_TEST_ON:
@@ -267,13 +312,8 @@ drawing_area_expose_event (GtkWidget      *widget,
 		break;
 	}
 
-	gdk_pixbuf_render_to_drawable_alpha
-		(pixbuf, widget->window,
-		 0, 0,
-		 (widget->allocation.width - gdk_pixbuf_get_width (pixbuf)) / 2,
-		 (widget->allocation.height - gdk_pixbuf_get_height (pixbuf)) / 2,
-		 -1, -1, GDK_PIXBUF_ALPHA_FULL, 0, GDK_RGB_DITHER_NORMAL, 0, 0);
-		
+	gtk_image_set_from_pixbuf (GTK_IMAGE (image), pixbuf);
+
 	return TRUE;
 }
 
@@ -309,27 +349,22 @@ load_pixbufs (void)
 
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "mouse-left.png", TRUE, NULL);
 	left_handed_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_object_add_weak_pointer (G_OBJECT (left_handed_pixbuf), (gpointer *) &left_handed_pixbuf);
 	g_free (filename);
 
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "mouse-right.png", TRUE, NULL);
 	right_handed_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_object_add_weak_pointer (G_OBJECT (right_handed_pixbuf), (gpointer *) &right_handed_pixbuf);
 	g_free (filename);
 
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "double-click-on.png", TRUE, NULL);
 	double_click_on_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_object_add_weak_pointer (G_OBJECT (double_click_on_pixbuf), (gpointer *) &double_click_on_pixbuf);
 	g_free (filename);
 
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "double-click-maybe.png", TRUE, NULL);
 	double_click_maybe_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_object_add_weak_pointer (G_OBJECT (double_click_maybe_pixbuf), (gpointer *) &double_click_maybe_pixbuf);
 	g_free (filename);
 
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "double-click-off.png", TRUE, NULL);
 	double_click_off_pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
-	g_object_add_weak_pointer (G_OBJECT (double_click_off_pixbuf), (gpointer *) &double_click_off_pixbuf);
 	g_free (filename);
 
 	called = TRUE;
@@ -342,7 +377,7 @@ read_cursor_font (void)
 	gchar *dir_name;
 	struct dirent *file_dirent;
 
-	dir_name = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".gnome/share/cursor-fonts", NULL);
+	dir_name = g_build_path (G_DIR_SEPARATOR_S, g_get_home_dir (), ".gnome2/share/cursor-fonts", NULL);
 	if (! g_file_test (dir_name, G_FILE_TEST_EXISTS))
 		return NULL;
 
@@ -485,8 +520,11 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	gconf_value_free (value);
 
 	/* Double-click time */
-	g_signal_connect (WID ("double_click_darea"), "expose_event", (GCallback) drawing_area_expose_event, changeset);
-
+	gtk_image_set_from_pixbuf (GTK_IMAGE (WID ("double_click_image")), double_click_off_pixbuf);
+	g_object_set_data (G_OBJECT (WID ("double_click_eventbox")), "image", WID ("double_click_image"));
+	g_signal_connect (WID ("double_click_eventbox"), "button_press_event",
+			  G_CALLBACK (event_box_button_press_event), changeset);
+	
 	/* Cursors page */
 	tree_view = WID ("cursor_tree");
 	cursor_font = read_cursor_font ();
@@ -532,7 +570,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "mouse-cursor-white.png", TRUE, NULL);
 	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 	g_free (filename);
-	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-white.pcf.gz", FALSE, NULL);
+	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-white.pcf", FALSE, NULL);
 
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	if (cursor_font && ! strcmp (cursor_font, font_path)) {
@@ -553,7 +591,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "mouse-cursor-normal-large.png", TRUE, NULL);
 	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 	g_free (filename);
-	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-large.pcf.gz", FALSE, NULL);
+	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-large.pcf", FALSE, NULL);
 
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	if (cursor_font && ! strcmp (cursor_font, font_path)) {
@@ -574,7 +612,7 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	filename = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_APP_PIXMAP, "mouse-cursor-white-large.png", TRUE, NULL);
 	pixbuf = gdk_pixbuf_new_from_file (filename, NULL);
 	g_free (filename);
-	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-large-white.pcf.gz", FALSE, NULL);
+	font_path = gnome_program_locate_file (program, GNOME_FILE_DOMAIN_DATADIR, "gnome/cursor-fonts/cursor-large-white.pcf", FALSE, NULL);
 
 	gtk_list_store_append (GTK_LIST_STORE (model), &iter);
 	if (cursor_font && ! strcmp (cursor_font, font_path)) {
@@ -618,13 +656,13 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	/* DnD threshold */
 	gconf_peditor_new_numeric_range
 		(changeset, "/desktop/gnome/peripherals/mouse/drag_threshold", WID ("drag_threshold_scale"),
-		 "conv-to-widget-cb", threshold_from_gconf,
-		 "conv-from-widget-cb", threshold_to_gconf,
+		 "conv-to-widget-cb", drag_threshold_from_gconf,
+		 "conv-from-widget-cb", drag_threshold_to_gconf,
 		 NULL);
 
 	/* listen to cursors changing */
         gconf_client_notify_add (gconf_client_get_default (),
-				 CURSOR_FONT_KEY, /* dir or key to listen to */
+				 CURSOR_FONT_KEY,
 				 cursor_font_changed,
 				 tree_view, NULL, NULL);
 
@@ -641,7 +679,7 @@ create_dialog (void)
 	GladeXML     *dialog;
 	GtkSizeGroup *size_group;
 
-	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-mouse-properties.glade", "prefs_widget", NULL);
+	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-mouse-properties.glade", "mouse_properties_dialog", NULL);
 	widget = glade_xml_get_widget (dialog, "prefs_widget");
 
 	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
@@ -659,22 +697,20 @@ create_dialog (void)
 	gtk_size_group_add_widget (size_group, WID ("slow_label"));
 	gtk_size_group_add_widget (size_group, WID ("small_label"));
 
-	g_object_weak_ref (G_OBJECT (widget), (GWeakNotify) g_object_unref, dialog);
-
 	return dialog;
 }
 
 /* Callback issued when a button is clicked on the dialog */
 
 static void
-dialog_button_clicked_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changeset) 
+dialog_response_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changeset) 
 {
-	switch (response_id) {
-	case GTK_RESPONSE_CLOSE:
-	default:
+	if (response_id == GTK_RESPONSE_HELP)
+		capplet_help (GTK_WINDOW (dialog),
+			"wgoscustdesk.xml",
+			"goscustperiph-5");
+	else
 		gtk_main_quit ();
-		break;
-	}
 }
 
 int
@@ -696,7 +732,8 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (argv[0], VERSION, LIBGNOMEUI_MODULE, argc, argv,
+	gnome_program_init ("gnome-mouse-properties", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PARAM_POPT_TABLE, cap_options,
 			    GNOME_PARAM_APP_DATADIR, GNOMECC_DATA_DIR,
 			    NULL);
@@ -714,13 +751,10 @@ main (int argc, char **argv)
 		load_pixbufs ();
 		setup_dialog (dialog, changeset);
 
-		dialog_win = gtk_dialog_new_with_buttons
-			(_("Mouse Properties"), NULL, -1,
-			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-			 NULL);
+		dialog_win = WID ("mouse_properties_dialog");
+		g_signal_connect (dialog_win, "response",
+				  G_CALLBACK (dialog_response_cb), changeset);
 
-		g_signal_connect (G_OBJECT (dialog_win), "response", (GCallback) dialog_button_clicked_cb, changeset);
-		gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog_win)->vbox), WID ("prefs_widget"), TRUE, TRUE, GNOME_PAD_SMALL);
 		gtk_widget_show_all (dialog_win);
 
 		gtk_main ();
