@@ -122,11 +122,12 @@ get_legacy_settings (void)
 static void
 dialog_button_clicked_cb (GtkDialog *dialog, gint response_id, GConfChangeSet *changeset) 
 {
-	switch (response_id) {
-	case GTK_RESPONSE_CLOSE:
+	if (response_id == GTK_RESPONSE_HELP)
+		capplet_help (GTK_WINDOW (dialog),
+			"wgoscustdesk.xml",
+			"goscustmulti-2");
+	else
 		gtk_main_quit ();
-		break;
-	}
 }
 
 int
@@ -153,7 +154,8 @@ main (int argc, char **argv)
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_program_init (argv[0], VERSION, LIBGNOMEUI_MODULE, argc, argv,
+	gnome_program_init ("gnome-sound-properties", VERSION,
+			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PARAM_POPT_TABLE, cap_options,
 			    NULL);
 
@@ -171,11 +173,12 @@ main (int argc, char **argv)
 
 #if 0
 		gnome_window_icon_set_default_from_file
-			(GNOMECC_ICONS_DIR "keyboard-capplet.png");
+			(GNOMECC_ICONS_DIR "sound-capplet.png");
 #endif
 
 		dialog_win = gtk_dialog_new_with_buttons
 			(_("Sound preferences"), NULL, -1,
+			 GTK_STOCK_HELP, GTK_RESPONSE_HELP,
 			 GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 			 NULL);
 
