@@ -40,12 +40,14 @@
 
 #define MONITOR_CONTENTS_X 0 
 #define MONITOR_CONTENTS_Y 0
-#define MONITOR_CONTENTS_WIDTH 64
-#define MONITOR_CONTENTS_HEIGHT 48
+#define MONITOR_CONTENTS_DEFAULT_WIDTH 64
+#define MONITOR_CONTENTS_DEFAULT_HEIGHT 48
 
 enum {
 	PROP_0,
-	PROP_TYPE
+	PROP_TYPE,
+	PROP_PREVIEW_WIDTH,
+	PROP_PREVIEW_HEIGHT
 };
 
 struct _BGApplierPrivate 
@@ -125,7 +127,9 @@ static void bg_applier_finalize      (GObject           *object);
 
 static void run_render_pipeline      (BGApplier           *bg_applier, 
 				      const BGPreferences *prefs);
-static void draw_disabled_message    (GtkWidget         *widget);
+static void draw_disabled_message    (GtkWidget         *widget,
+				      const guint width,
+				      const guint height);
 
 static void render_background        (BGApplier           *bg_applier,
 				      const BGPreferences *prefs);
@@ -174,10 +178,13 @@ static gboolean is_nautilus_running  (void);
 
 static gboolean cleanup_cb           (BGApplier *bg_applier);
 
-guint
+static void preview_realized_cb      (GtkWidget *preview,
+				      BGApplier *bg_applier);
+
+GType
 bg_applier_get_type (void)
 {
-	static guint bg_applier_type = 0;
+	static GType bg_applier_type = 0;
 
 	if (!bg_applier_type) {
 		static GTypeInfo bg_applier_info = {
@@ -210,6 +217,8 @@ bg_applier_init (BGApplier *bg_applier, BGApplierClass *class)
 	bg_applier->p->pixbuf           = NULL;
 	bg_applier->p->wallpaper_pixbuf = NULL;
 	bg_applier->p->timeout          = 0;
+	bg_applier->p->render_geom.width = -1;
+	bg_applier->p->render_geom.height = -1;
 }
 
 static void
@@ -232,6 +241,22 @@ bg_applier_class_init (BGApplierClass *class)
 				   0, 1, 0,
 				   G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
 
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_WIDTH,
+		 g_param_spec_uint ("preview_width",
+			 	    _("Preview Width"),
+				    _("Width if applier is a preview: Defaults to 64."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_WIDTH,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
+	g_object_class_install_property
+		(object_class, PROP_PREVIEW_HEIGHT,
+		 g_param_spec_uint ("preview_height",
+				    _("Preview Height"),
+				    _("Height if applier is a preview: Defaults to 48."),
+				    1, 65535, MONITOR_CONTENTS_DEFAULT_HEIGHT,
+				    G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
 	parent_class = 
 		G_OBJECT_CLASS (g_type_class_ref (G_TYPE_OBJECT));
 }
@@ -268,8 +293,13 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 		case BG_APPLIER_PREVIEW:
 			bg_applier->p->render_geom.x = MONITOR_CONTENTS_X;
 			bg_applier->p->render_geom.y = MONITOR_CONTENTS_Y;
-			bg_applier->p->render_geom.width = MONITOR_CONTENTS_WIDTH;
-			bg_applier->p->render_geom.height = MONITOR_CONTENTS_HEIGHT;
+			
+			if (bg_applier->p->render_geom.width == -1)
+			{
+				bg_applier->p->render_geom.width = MONITOR_CONTENTS_DEFAULT_WIDTH;
+				bg_applier->p->render_geom.height = MONITOR_CONTENTS_DEFAULT_HEIGHT;
+			}
+
 			break;
 
 		default:
@@ -279,6 +309,16 @@ bg_applier_set_prop (GObject *object, guint prop_id, const GValue *value, GParam
 
 		break;
 
+	case PROP_PREVIEW_WIDTH:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.width = g_value_get_uint (value);
+		break;
+
+	case PROP_PREVIEW_HEIGHT:
+		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
+			bg_applier->p->render_geom.height = g_value_get_uint (value);
+		break;
+		
 	default:
 		g_warning ("Bad property set");
 		break;
@@ -354,6 +394,22 @@ bg_applier_new (BGApplierType type)
 	return object;
 }
 
+GObject *
+bg_applier_new_at_size (BGApplierType type,
+			const guint width,
+			const guint height)
+{
+	GObject *object;
+
+	object = g_object_new (bg_applier_get_type (),
+			       "type", type,
+			       "preview_width", width,
+			       "preview_height", height,
+			       NULL);
+
+	return object;
+}
+
 void
 bg_applier_apply_prefs (BGApplier           *bg_applier, 
 			const BGPreferences *prefs)
@@ -372,13 +428,12 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 	}
 
 	if (bg_applier->p->type == BG_APPLIER_ROOT && is_nautilus_running ()) {
-		set_root_pixmap ((GdkPixmap *) -1);
 		return;
 	}
 
 	if (!new_prefs->enabled) {
 		if (bg_applier->p->type == BG_APPLIER_PREVIEW)
-			draw_disabled_message (bg_applier_get_preview_widget (bg_applier));
+			draw_disabled_message (bg_applier_get_preview_widget (bg_applier), bg_applier->p->render_geom.width, bg_applier->p->render_geom.height);
 		return;
 	}
 
@@ -395,8 +450,6 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 				gdk_pixbuf_new_from_file (new_prefs->wallpaper_filename, NULL);
 
 			if (bg_applier->p->wallpaper_pixbuf == NULL) {
-				g_warning (_("Could not load pixbuf \"%s\"; disabling wallpaper."),
-					   new_prefs->wallpaper_filename);
 				new_prefs->wallpaper_enabled = FALSE;
 			}
 			else if (bg_applier->p->type == BG_APPLIER_ROOT) {
@@ -407,9 +460,6 @@ bg_applier_apply_prefs (BGApplier           *bg_applier,
 		}
 	}
 
-	if (bg_applier->p->type == BG_APPLIER_ROOT)
-		nice (20);
-
 	run_render_pipeline (bg_applier, new_prefs);
 
 	if (bg_applier->p->last_prefs != NULL)
@@ -437,11 +487,16 @@ bg_applier_get_preview_widget (BGApplier *bg_applier)
 {
 	if (bg_applier->p->preview_widget == NULL)
 	{
-		GdkPixmap *pixmap;
+		bg_applier->p->preview_widget = gtk_image_new ();
 
-		pixmap = gdk_pixmap_new (gdk_get_default_root_window (), MONITOR_CONTENTS_WIDTH, MONITOR_CONTENTS_HEIGHT, -1);
-		bg_applier->p->preview_widget = gtk_image_new_from_pixmap (pixmap, NULL);
+		/* We need to initialize the pixmap, but this
+		 * needs GCs, so we have to wait until realize. */
+		g_signal_connect (G_OBJECT (bg_applier->p->preview_widget),
+				  "realize",
+				  (GCallback) preview_realized_cb,
+				  bg_applier);
 	}
+
 	return bg_applier->p->preview_widget;
 }
 
@@ -455,21 +510,19 @@ bg_applier_get_wallpaper_pixbuf (BGApplier *bg_applier)
 }
 
 static void
-draw_disabled_message (GtkWidget *widget)
+draw_disabled_message (GtkWidget *widget, const guint w, const guint h)
 {
 	GdkPixmap      *pixmap;
 	GdkColor        color;
 	PangoLayout    *layout;
 	PangoRectangle  extents;
 	GdkGC          *gc;
-	gint            x, y, w, h;
+	gint            x, y;
 	const char     *disabled_string = _("Disabled");
 
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GTK_IS_IMAGE (widget));
 
-	w = MONITOR_CONTENTS_WIDTH;
-	h = MONITOR_CONTENTS_HEIGHT;
 	x = MONITOR_CONTENTS_X;
 	y = MONITOR_CONTENTS_Y;
 
@@ -708,7 +761,7 @@ render_to_screen (BGApplier *bg_applier, const BGPreferences *prefs)
 	g_return_if_fail (prefs != NULL);
 	g_return_if_fail (IS_BG_PREFERENCES (prefs));
 
-	gc = gdk_gc_new (gdk_get_default_root_window ());
+	gc = gdk_gc_new (bg_applier->p->pixmap);
 
 	if (bg_applier->p->pixbuf != NULL) {
 		if (bg_applier->p->pixbuf_render_geom.x != 0 ||
@@ -827,7 +880,8 @@ create_pixmap (BGApplier *bg_applier, const BGPreferences *prefs)
 #if 0
 		bg_applier->p->pixmap = GTK_PIXMAP (bg_applier->p->preview_widget)->pixmap;
 #else
-		gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
+		if (!bg_applier->p->pixmap)
+			gtk_image_get_pixmap (GTK_IMAGE (bg_applier->p->preview_widget), &bg_applier->p->pixmap, NULL);
 #endif
 		bg_applier->p->pixmap_is_set = TRUE;
 		break;
@@ -1211,7 +1265,9 @@ need_wallpaper_load_p (const BGApplier *bg_applier, const BGPreferences *prefs)
 static gboolean
 need_root_pixmap_p (const BGApplier *bg_applier, const BGPreferences *prefs) 
 {
-	if (prefs->wallpaper_enabled == FALSE && prefs->gradient_enabled == FALSE)
+	if (bg_applier->p->pixmap == NULL)
+		return TRUE;
+	else if (prefs->wallpaper_enabled == FALSE && prefs->gradient_enabled == FALSE)
 		return FALSE;
 	else if (bg_applier->p->last_prefs == NULL)
 		return TRUE;
@@ -1288,8 +1344,9 @@ make_root_pixmap (gint width, gint height)
 {
 	Display *display;
 	Pixmap xpixmap;
+	GdkPixmap *gdkpixmap;
 	
-	display = XOpenDisplay (g_getenv ("DISPLAY"));
+	display = XOpenDisplay (gdk_get_display ());
 	XSetCloseDownMode (display, RetainPermanent);
 
 	xpixmap = XCreatePixmap (display,
@@ -1299,7 +1356,11 @@ make_root_pixmap (gint width, gint height)
 
 	XCloseDisplay (display);
 
-	return gdk_pixmap_foreign_new (xpixmap);
+	gdkpixmap = gdk_pixmap_foreign_new (xpixmap);
+	gdk_drawable_set_colormap (GDK_DRAWABLE (gdkpixmap),
+				   gdk_drawable_get_colormap (gdk_get_default_root_window ()));
+
+	return gdkpixmap;
 }
 
 /* Set the root pixmap, and properties pointing to it. We
@@ -1318,6 +1379,11 @@ set_root_pixmap (GdkPixmap *pixmap)
 	guchar *data_esetroot;
 	Pixmap pixmap_id;
 
+	/* Final check to see if nautilus is running. If it is, we don't
+	   touch the root pixmap at all. */
+	if (is_nautilus_running ())
+		return;
+
 	if (pixmap != NULL && pixmap != (GdkPixmap *) -1)
 		pixmap_id = GDK_WINDOW_XWINDOW (pixmap);
 	else
@@ -1453,3 +1519,31 @@ cleanup_cb (BGApplier *bg_applier)
 	
 	return FALSE;
 }
+
+static void
+preview_realized_cb (GtkWidget *preview, BGApplier *bg_applier)
+{
+	GdkPixmap *pixmap;
+
+	/* Only draw clean image if no pref set yet */
+	if (bg_applier->p->last_prefs)
+		return;       
+
+	gtk_image_get_pixmap (GTK_IMAGE (preview), &pixmap, NULL);
+	if (!pixmap) {
+		pixmap = gdk_pixmap_new (preview->window,
+					 bg_applier->p->render_geom.width,
+					 bg_applier->p->render_geom.height,
+					 -1);
+		gtk_image_set_from_pixmap (GTK_IMAGE (preview), pixmap, NULL);
+	}
+	
+	gdk_draw_rectangle (pixmap,
+			    preview->style->bg_gc[preview->state],
+			    TRUE,
+			    bg_applier->p->render_geom.x,
+			    bg_applier->p->render_geom.y,
+			    bg_applier->p->render_geom.width,
+			    bg_applier->p->render_geom.height);
+}
+
